/* ======================================================================== *
 * MATHLIB -- TI Floating-Point Math Function Library                       *
 *                                                                          *
 *                                                                          *
 * Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/   *
 *                                                                          *
 *                                                                          *
 *  Redistribution and use in source and binary forms, with or without      *
 *  modification, are permitted provided that the following conditions      *
 *  are met:                                                                *
 *                                                                          *
 *    Redistributions of source code must retain the above copyright        *
 *    notice, this list of conditions and the following disclaimer.         *
 *                                                                          *
 *    Redistributions in binary form must reproduce the above copyright     *
 *    notice, this list of conditions and the following disclaimer in the   *
 *    documentation and/or other materials provided with the                *
 *    distribution.                                                         *
 *                                                                          *
 *    Neither the name of Texas Instruments Incorporated nor the names of   *
 *    its contributors may be used to endorse or promote products derived   *
 *    from this software without specific prior written permission.         *
 *                                                                          *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     *
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR   *
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT    *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   *
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   *
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     *
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    *
 * ======================================================================== */

/* ======================================================================= */
/* sinhsp_d.h - hyperbolic sine driver file                                */
/* ======================================================================= */

#include <stdio.h>
#include <math.h>

#include <ti/mathlib/src/common/drvsp.h>
#include "sinhsp.h"


/* ======================================================================== */
/*  Defines for driver test functionality                                   */
/* ======================================================================== */

/* Range for random inputs:  RAND_OFFSET to (RAND_RANGE + RAND_OFFSET) */
#define RAND_RANGE   (2*95.0)
#define RAND_OFFSET  ( -95.0)

/*  Number of input arguments */
#define NUM_INPUTS     (201)
#define NUM_SC_INPUTS    (8)
#define NUM_EXT_INPUTS (125)

/* Floating Point Tolerances */
#define RET_VAL_TOL   (1e-6)

/* Number of random test values */
#define BUF_SIZE       (128)

/* Function name */
#define KERNEL_NAME  "sinhSP"

/* Equality comparison requirements */
#define KERNEL_REQ   (MTH_REQ_PERCENT|MTH_REQ_IGNORE_INF)


/* ======================================================================== */
/*  Pre-defined input test data                                             */
/* ======================================================================== */

float a [NUM_INPUTS] =
{
/*   0 -  09 */  -1.192093e-07, 1.319647e-07, -1.460849e-07, 1.617160e-07, -1.790196e-07, 1.981747e-07, -2.193794e-07, 2.428530e-07, -2.688382e-07, 2.976039e-07,
/*  10 -  19 */  -3.294476e-07, 3.646985e-07, -4.037212e-07, 4.469194e-07, -4.947397e-07, 5.476769e-07, -6.062783e-07, 6.711501e-07, -7.429632e-07, 8.224602e-07,
/*  20 -  29 */  -9.104634e-07, 1.007883e-06, -1.115726e-06, 1.235109e-06, -1.367266e-06, 1.513563e-06, -1.675515e-06, 1.854795e-06, -2.053258e-06, 2.272956e-06,
/*  30 -  39 */  -2.516163e-06, 2.785392e-06, -3.083429e-06, 3.413356e-06, -3.778585e-06, 4.182894e-06, -4.630464e-06, 5.125923e-06, -5.674397e-06, 6.281557e-06,
/*  40 -  49 */  -6.953684e-06, 7.697728e-06, -8.521385e-06, 9.433173e-06, -1.044252e-05, 1.155987e-05, -1.279678e-05, 1.416603e-05, -1.568180e-05, 1.735975e-05,
/*  50 -  59 */  -1.921725e-05, 2.127349e-05, -2.354975e-05, 2.606958e-05, -2.885902e-05, 3.194694e-05, -3.536526e-05, 3.914934e-05, -4.333832e-05, 4.797553e-05,
/*  60 -  69 */  -5.310891e-05, 5.879156e-05, -6.508225e-05, 7.204606e-05, -7.975499e-05, 8.828877e-05, -9.773567e-05, 0.0001081934, -0.0001197701, 0.0001325855,
/*  70 -  79 */  -0.0001467721, 0.0001624767, -0.0001798617, 0.0001991070, -0.0002204114, 0.0002439954, -0.0002701029, 0.0002990039, -0.0003309974, 0.0003664141,
/*  80 -  89 */  -0.0004056204, 0.0004490218, -0.0004970671, 0.0005502533, -0.0006091304, 0.0006743073, -0.0007464582, 0.0008263292, -0.0009147465, 0.001012624,
/*  90 -  99 */  -0.001120975,  0.001240919,  -0.001373698,  0.001520684,  -0.001683397,  0.001863520,  -0.002062917,  0.002283649,  -0.002527999,  0.002798495,
/* 100 - 109 */  -0.003097934,  0.003429413,  -0.003796360,  0.004202571,  -0.004652246,  0.005150036,  -0.005701090,  0.006311107,  -0.006986395,  0.007733940,
/* 110 - 119 */  -0.008561471,  0.009477548,  -0.01049165,   0.01161425,   -0.01285698,   0.01423267,   -0.01575557,   0.01744142,   -0.01930765,   0.02137357,
/* 120 - 129 */  -0.02366054,   0.02619221,   -0.02899478,   0.03209722,   -0.03553163,   0.03933351,   -0.04354220,   0.04820121,   -0.05335874,   0.05906812,
/* 130 - 139 */  -0.06538842,   0.07238498,   -0.08013017,   0.08870409,   -0.09819543,   0.1087023,    -0.1203335,    0.1332092,    -0.1474626,    0.1632411,
/* 140 - 149 */  -0.1807079,    0.2000436,    -0.2214483,    0.2451432,    -0.2713735,    0.3004105,    -0.3325544,    0.3681378,    -0.4075285,    0.4511341,
/* 150 - 159 */  -0.4994054,    0.5528418,    -0.6119958,    0.6774794,    -0.7499697,    0.8302165,    -0.9190496,    1.017388,     -1.126248,     1.246757,
/* 160 - 169 */  -1.380160,     1.527837,     -1.691316,     1.872286,     -2.072621,     2.294392,     -2.539891,     2.811660,     -3.112508,     3.445546,
/* 170 - 179 */  -3.814219,     4.222341,     -4.674131,     5.174263,     -5.727909,     6.340796,     -7.019261,     7.770322,     -8.601746,     9.522133,
/* 180 - 189 */  -10.54100,     11.66889,     -12.91746,     14.29963,     -15.82969,     17.52346,     -19.39848,     21.47411,     -23.77184,     26.31543,
/* 190 - 199 */  -29.13118,     32.24821,     -35.69878,     39.51854,     -43.74703,     48.42796,     -53.60975,     59.34599,     -65.69601,     72.72549,
/* 200 - 200 */  -80.50712,
};

/* This function saves the result obtained from predefined data to the specified file*/
/*
void save_output(float * output, int size)
{
  int i;
  FILE * out = fopen("../sinhsp_output.dat","w");

  for (i = 0; i < size; i++) {
    fprintf(out,"%.7e\n",output[i]);
  }
  fclose(out);
}
*/

/* ======================================================================== */
/*  Main test driver                                                        */
/* ======================================================================== */

void main()
{
  int i;  

  /* ======================================================================== */
  /*  INITIALIZATION                                                          */
  /* ======================================================================== */

  /* Initialize the driver */
  driver_init (KERNEL_NAME);

  /* ======================================================================== */
  /*  PREDEFINED TEST DATA                                                    */
  /* ======================================================================== */

  /* Obtain results using predefined test vectors */
  for (i = 0; i < NUM_INPUTS ; i++) {
    output[MTH_FCN_RTS][i] = sinhf (a[i]);
  }
  for (i = 0; i < NUM_INPUTS ; i++) {
    output[MTH_FCN_ASM][i] = sinhsp (a[i]);
  }
  for (i = 0; i < NUM_INPUTS; i++) {
    output[MTH_FCN_CI][i] = sinhsp_c (a[i]);
  }
  for (i = 0; i < NUM_INPUTS ; i++) {
    output[MTH_FCN_INL][i] = sinhsp_i (a[i]);
  }
  sinhsp_v (a, output[MTH_FCN_VEC], NUM_INPUTS);

//  save_output(output[MTH_FCN_INL], NUM_INPUTS);

  /* Check results and display */
  allequal (NUM_INPUTS, RET_VAL_TOL, KERNEL_REQ);
  print_test_results (MTH_TST_PREDEF);

  /* ======================================================================== */
  /*  SPECIAL CASE and EXTENDED RANGE TEST DATA                               */
  /* ======================================================================== */

  /* Obtain results using special case test vectors */
  a_sc[0] = 0;
  a_sc[1] = _itof(0x80000000);            /* -0.0 */
  a_sc[2] = _itof(0x7F800000);            /* INF */
  a_sc[3] = _itof(0xFF800000);            /* -INF */
  a_sc[4] = 1;
  a_sc[5] = -1;
  a_sc[6] = _itof(0x7FFFFFFF);            /* NaN */
  a_sc[7] = _itof(0xFFFFFFFF);            /* -NaN */

  for (i = 0; i < NUM_SC_INPUTS ; i++) {
    output[MTH_FCN_RTS][i] = sinhf (a_sc[i]);
    output[MTH_FCN_ASM][i] = sinhsp (a_sc[i]);
    output[MTH_FCN_CI] [i] = sinhsp_c (a_sc[i]);
    output[MTH_FCN_INL][i] = sinhsp_i (a_sc[i]);
  }
  sinhsp_v (a_sc, output[MTH_FCN_VEC], NUM_SC_INPUTS);

  /* Check results and display */
  allequal (NUM_SC_INPUTS, RET_VAL_TOL, KERNEL_REQ);
  print_test_results (MTH_TST_SPECIAL);

  /* Obtain results using extended range test vectors */
  a_ext[0] = FLT_MIN;
  for (i = 0; i < NUM_EXT_INPUTS ; i++) {
    output[MTH_FCN_RTS][i] = sinhf (a_ext[i]);
    output[MTH_FCN_ASM][i] = sinhsp (a_ext[i]);
    output[MTH_FCN_CI] [i] = sinhsp_c (a_ext[i]);
    output[MTH_FCN_INL][i] = sinhsp_i (a_ext[i]);
    a_ext[i+1] = a_ext[i] * (-2.1);
  }
  sinhsp_v (a_ext, output[MTH_FCN_VEC], NUM_EXT_INPUTS);

  /* Check results and display */
  allequal (NUM_EXT_INPUTS, RET_VAL_TOL, KERNEL_REQ);
  print_test_results (MTH_TST_EXTENDED);

  /* ======================================================================== */
  /*  RANDOM TEST DATA & PROFILING                                            */
  /* ======================================================================== */

  // Time the fastRTS inline function
  profile_init (MTH_FCN_INL);
  for (i = 0; i < BUF_SIZE; i++) {
    /* Fill input random data vector */
    input[0][i] = gimme_random (RAND_RANGE, RAND_OFFSET);
    
    /* Run the inline routine */
    profile_start ();
    output[MTH_FCN_INL][i] = sinhsp_i (input[0][i]);
    profile_update (MTH_FCN_INL);
  }
  profile_avg (MTH_FCN_INL, BUF_SIZE);
  
  // Time the RTS function 
  profile_init (MTH_FCN_RTS);
  for (i = 0; i < BUF_SIZE; i++) {
    profile_start();
    output[MTH_FCN_RTS][i] = sinhf (input[0][i]);
    profile_update (MTH_FCN_RTS);
  }
  profile_avg    (MTH_FCN_RTS, BUF_SIZE);

  // Time the fastRTS C intrinsic function
  profile_init (MTH_FCN_CI);
  for (i = 0; i < BUF_SIZE; i++) {
    profile_start();
    output[MTH_FCN_CI][i] = sinhsp_c (input[0][i]);
    profile_update (MTH_FCN_CI);
  }
  profile_avg    (MTH_FCN_CI, BUF_SIZE);

  // Time the fastRTS ASM function
  profile_init (MTH_FCN_ASM);
  for (i = 0; i < BUF_SIZE; i++) {
    profile_start();
    output[MTH_FCN_ASM][i] = sinhsp (input[0][i]);
    profile_update (MTH_FCN_ASM);
  }
  profile_avg    (MTH_FCN_ASM, BUF_SIZE);

  // Time the fastRTS vector function
  profile_init (MTH_FCN_VEC);
  sinhsp_v (input[0], output[MTH_FCN_VEC], BUF_SIZE);
  profile_update (MTH_FCN_VEC);
  profile_avg    (MTH_FCN_VEC, BUF_SIZE);

  /* Display Random Test Vector Results */
  allequal (BUF_SIZE, RET_VAL_TOL, KERNEL_REQ);
  print_test_results (MTH_TST_RANDOM);
  print_hline ();
  
  /* ======================================================================== */
  /*  CYCLE AND MEMORY REQUIREMENTS                                           */
  /* ======================================================================== */

  /* Display profiling results */
  print_profile_results (KERNEL_NAME);

  /* Display memory requirements */
  print_memory_results (KERNEL_NAME);
}

/* ======================================================================== */
/*  End of file: sinhsp_d.h                                                 */
/* ======================================================================== */
